@@ -1,6 +1,6 @@
 #!/usr/bin/perl -T
 ###
-###              analog 5.03             http://www.analog.cx/
+###              analog 5.1             http://www.analog.cx/
 ### This program is copyright (c) Stephen R. E. Turner 1995 - 2001 except as
 ### stated otherwise. Distribution, usage and modification of this program is
 ### subject to the conditions of the Licence which you should have received
@@ -13,8 +13,8 @@
 
 # 1) uncomment (remove everything before $analog) and edit one of the next two
 #    lines to give the location (full pathname) of the analog executable.
-#       Unix: $analog = '/usr/local/etc/httpd/analog-5.03/analog';
-#       Windows: $analog = 'C:\program files\analog 5.03\analog.exe';
+#       Unix: $analog = '/usr/local/etc/httpd/analog-5.1/analog';
+#       Windows: $analog = 'C:\program files\analog 5.1\analog.exe';
 # 2) If you're on Unix, edit the first line in this file to give the location
 #    of Perl (don't remove the #! though).
 # 3) You also need to edit anlgform.html if you want to use the form.
@@ -30,12 +30,16 @@
 # If you add a command, you must also add any aliases it possesses.
 # There is a discussion of all this in docs/form.html.
 
+@allowed = qw();
+# Allowed commands. If there are _any_ commands listed here, then _only_
+# commands which are in @allowed, and not in @forbidden, can be used.
+
 require 5.001;
 use CGI;
 
 # 1) INITIALISATION
 # delete all dangerous environment variables
-$ENV{PATH} = '';  # blank not deleted so that UNCOMPRESS doesn't get a path
+$ENV{PATH} = '';  # blank, not deleted, so that UNCOMPRESS doesn't get a path
 delete @ENV{qw/IFS CDPATH ENV BASH_ENV/};
 $query = new CGI;
 $|=1;
@@ -61,7 +65,7 @@ checkfilechars("CACHEFILE");
 
 # 2) OPEN THE ANALOG PROCESS
 # qv=1 causes args to go straight to stdout, not program
-if (${$args{'QV'}}[-1] && !grep('QV' eq $_, @forbidden)) {
+if (${$args{'QV'}}[-1] && !forbidden('QV')) {
     print "Content-Type: text/plain\n\n";
     open(ANALOG, ">-");
 }
@@ -86,21 +90,22 @@ else {
 
 # 3) PRINT ALL THE COMMANDS
 # Special cases: must come first
-printargs('CG', 'CONFIGFILE') unless grep('CG' eq $_, @forbidden);
+printargs('CG', 'CONFIGFILE') unless forbidden('CG');
+# both 'CG' and 'CONFIGFILE' must be allowed for this to happen.
 print ANALOG "CGI ON\nDNS NONE\nWARNINGS FL\n";
-printargs('WARNINGS') unless grep('WARNINGS' eq $_, @forbidden);
-printargs('LOGTIMEOFFSET') unless grep('LOGTIMEOFFSET' eq $_, @forbidden);
+printargs('WARNINGS');
+printargs('LOGTIMEOFFSET');
 foreach $k (@order) {
     printargs($k)
         unless($k eq 'QV' || $k eq 'CG' || $k eq 'CM' || $k =~ /FLOORB$/ ||
 	       $k =~ /2$/ || $k =~ '^LOGTIMEOFFSET' || $k =~ '^WARNINGS' ||
                # commands dealt with elsewhere
-               $k =~ /[^A-Z12]/ || $k eq '' ||
-	       grep($k =~ /^$_.?$/, @forbidden) ||
-	       $k =~ /^IGNORE/); # other stuff not wanted
+               $k =~ /[^A-Z12]/ || $k eq '' || $k =~ /^IGNORE/);
+               # other stuff not wanted
 }
 # Special case: must come last
-printargs('CM', 'CONFIGFILE') unless grep('CM' eq $_, @forbidden);
+printargs('CM', 'CONFIGFILE') unless forbidden('CM');
+# again, both 'CM' and 'CONFIGFILE' must be allowed for this to happen.
 print ANALOG "OUTFILE stdout\n";
 
 # 4) WAIT FOR PROCESS TO FINISH. THAT'S IT.
@@ -114,7 +119,15 @@ unless (close(ANALOG)) {
     die;
 }
 
-# A) PRINT ONE COMMAND
+
+### SUBROUTINES 
+# A) IS A GIVEN COMMAND FORBIDDEN?
+sub forbidden {
+    return (grep($_[0] eq $_, @forbidden) ||
+	    (@allowed && !grep($_[0] eq $_, @allowed)));
+}
+
+# B) PRINT ONE COMMAND
 sub printargs {
     my($is_floora) = 0;
     my($is_12) = 0;
@@ -122,23 +135,29 @@ sub printargs {
     if ($name =~ /FLOORA$/) {
 	chop($name);
 	$is_floora = 1;
-	$b = ${$args{$name . 'B'}}[-1];
     }
     elsif ($name =~ /1$/) {
 	chop($name);
 	$is_12 = 1;
-	$b = ${$args{$name . '2'}}[-1];
     }
-    foreach $a (@{$args{$_[0]}}) {
-	if ($a ne '') {
-	    if ($is_floora) {
-		print ANALOG ("$name $a$b\n") if ($b ne '');
-	    }
-	    elsif ($is_12) {
-		print ANALOG ("$name ", bracket($a), " ", bracket($b), "\n")
-		    if ($b ne '');
-	    }
-	    else {
+    return if forbidden($name);
+
+    if ($is_floora) {
+	$a = ${$args{$name . 'A'}}[-1];  # last "FLOORA=$a" form arg specified
+	$b = ${$args{$name . 'B'}}[-1];
+	print ANALOG ("$name $a$b\n") if ($b ne '' && $b !~ /\\$/);
+	# could bracket $a$b, but no help because any special character in a
+	# FLOOR command is junk anyway.
+    }
+    elsif ($is_12) {
+	$a = ${$args{$name . '1'}}[-1];
+	$b = ${$args{$name . '2'}}[-1];
+	print ANALOG ("$name ", bracket($a), " ", bracket($b), "\n")
+	    if ($b ne '');
+    }
+    else {
+	foreach $a (@{$args{$_[0]}}) {  # run through all "NAME=$a" form args
+	    if ($a ne '') {
 		print ANALOG ("$name ", bracket($a), "\n");
 		print ANALOG ("DNS READ\n") if ($name eq 'DNSFILE');
 	    }
@@ -146,7 +165,7 @@ sub printargs {
     }
 }
 
-# B) PUT APPROPRIATE DELIMITERS ROUND AN ARGUMENT CONTAINING SPACES
+# C) PUT APPROPRIATE DELIMITERS ROUND AN ARGUMENT CONTAINING SPACES
 sub bracket {
     local $_ = $_[0];
     return $_ unless (/[\s\#]/ || /^['"\(]/ || /\\$/);
@@ -156,7 +175,7 @@ sub bracket {
     # analog has no syntax if string contains ) as well as space, ' and "
 }
 
-# C) CHECK ONLY SAFE CHARACTERS in LOGFILEs and CACHEFILEs. See docs/form.html.
+# D) CHECK ONLY SAFE CHARACTERS in LOGFILEs and CACHEFILEs. See docs/form.html.
 sub checkfilechars {
     local ($_);
     foreach (@{$args{$_[0]}}, @{$args{$_[0] . '1'}}) {
@@ -170,7 +189,7 @@ sub checkfilechars {
     }
 }
 
-# D) CHECK NO UNSAFE CHARACTERS IN OTHER COMMANDS. Again, see docs/form.html.
+# E) CHECK NO UNSAFE CHARACTERS IN OTHER COMMANDS. Again, see docs/form.html.
 sub checkchars {
     local $_ = $_[0];
     if (/[\x00-\x1F\x7F-\x9F]/) {
@@ -180,7 +199,7 @@ sub checkchars {
     }
 }
 
-# E) PRINT OUT ERROR MESSAGE
+# F) PRINT OUT ERROR MESSAGE
 sub badreq {
     my($i);
     print "Content-Type: text/html\n";
